import Foundation
import UIKit
import QuickLook
import ARKit

public class SwiftFlutterR2uPlugin: NSObject, FlutterPlugin {
  var viewController: ViewController
  
  override init() {
    self.viewController = ViewController()
  }
  
  /**
   * See
   * https://gist.github.com/flutter-devs/be36131c0fe5a5bd9be6271bb1d07152
   * https://stackoverflow.com/questions/53898765/flutter-plugin-invoking-ios-and-android-method-including-parameters-not-working/53899965
   */
  
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "flutter_r2u", binaryMessenger: registrar.messenger())
    let instance = SwiftFlutterR2uPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }
  
  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    if (call.method == "deviceSupportsAR") {
      let supportsAR = deviceSupportsAR()
      result(supportsAR)
    } else if (call.method == "openAR") {
      guard let arguments = call.arguments else {
        result("iOS could not recognize flutter arguments in method: (openAR)")
        return
      }
      let args = arguments as? [String: Any]
      let usdz: String = args!["usdz"] as! String
      let name: String = args!["name"] as! String
      let resize: Bool = args!["resize"] as! Bool
      if #available(iOS 10.0, *) {
        openAR(url: usdz, name: name, resize: resize, result: result)
      } else {
        result(false)
      }
    } else {
      result(FlutterMethodNotImplemented)
    }
  }
  
  
  @available(iOS 10.0, *)
  func openAR(url: String,
              name: String,
              resize: Bool,
              result: @escaping FlutterResult) -> Void {
    self.viewController.url = url
    self.viewController.resize = resize
    let completion = { result(true) }
    let cameraMediaType = AVMediaType.video
    let cameraAuthorizationStatus = AVCaptureDevice.authorizationStatus(for: cameraMediaType)
    
    switch cameraAuthorizationStatus {
    case .denied:
      result(false)
      break
    case .authorized:
      self.viewController.quicklook(completion: completion)
      break;
    case .restricted:
      result(false)
      break
    case .notDetermined:
      // Prompting user for the permission to use the camera.
      AVCaptureDevice.requestAccess(for: cameraMediaType) { granted in
        if granted {
          self.viewController.quicklook(completion: completion)
        }
        else {
          result(false)
        }
      }
    @unknown default:
      result(false)
    }
  }
  
  func deviceSupportsAR() -> Bool {
    var isSimulator = false
    #if targetEnvironment(simulator)
    isSimulator = true
    #endif
    
    if #available(iOS 11.0, *) {
      return !isSimulator && ARConfiguration.isSupported
    } else {
      return false
    }
  }
}

class ViewController: UIViewController, QLPreviewControllerDelegate, QLPreviewControllerDataSource {
  let previewController = QLPreviewController()
  var url: String?
  var resize: Bool?
  
  var fileAt: URL?
  
  override func viewDidLoad() {
    super.viewDidLoad()
  }
  
  func topMostController() -> UIViewController {
    var topController: UIViewController = UIApplication.shared.keyWindow!.rootViewController!
    while (topController.presentedViewController != nil) {
      topController = topController.presentedViewController!
    }
    return topController
  }
  
  @available(iOS 10.0, *)
  func quicklook(completion: @escaping () -> Void) {
    let url = URL(string: self.url!)!
    URLSession.shared.dataTask(with: url) { data, response, error in
      do {
        let httpURLResponse = response as! HTTPURLResponse
        let suggestedFilename = httpURLResponse.suggestedFilename
        let previewURL = FileManager.default.temporaryDirectory.appendingPathComponent(suggestedFilename!)
        try data!.write(to: previewURL, options: .atomic)
        self.fileAt = previewURL
        DispatchQueue.main.async {
          UIApplication.shared.isNetworkActivityIndicatorVisible = false
          self.previewController.delegate = self
          self.previewController.dataSource = self
          self.previewController.currentPreviewItemIndex = 0
          let topVC = self.topMostController()
          topVC.present(self.previewController, animated: true, completion: completion)
        }
      } catch {
        print(error)
        return
      }
    }.resume()
    UIApplication.shared.isNetworkActivityIndicatorVisible = true
  }
  // https://github.com/dbystruev/ARQuickLook/blob/master/ARQuickLook/View%20Controllers/ViewController.swift
  func previewController(_ controller: QLPreviewController, previewItemAt index: Int) -> QLPreviewItem {
    let item = self.fileAt! as QLPreviewItem
    
    if #available(iOS 13.0, *) {
      let item = ARQuickLookPreviewItem(fileAt: self.fileAt!)
      item.allowsContentScaling = self.resize ?? false
      return item
    }
    
    return item
  }
  func numberOfPreviewItems(in controller: QLPreviewController) -> Int {
    1
  }
}
